"""
Postfix specific configuration files, the file locations are for ubuntu 14.04
"""
#import re

from . import generic
from . import base


class SpamAssassinOption(base.ConfigOption):
    def format_set(self, name, divider, val):
        option_set_format = "{}{}{}" # key=val
        s = option_set_format.format(name, divider, val)
        return s


class SpamAssassin(generic.EqualConfig):
    option_class = SpamAssassinOption
    dest_path = "/etc/default/spamassassin"


class Local(generic.SpaceConfig):
    dest_path = "/etc/spamassassin/local.cf"
    option_name_regexes = [
        "^score\s+[A-Z0-9_]+"
    ]

    def update_option(self, option, v):
        if option.name == "loadplugin":
            ret = False
            if option.val == v:
                option.val = v
                ret = True

        else:
            ret = super(Local, self).update_option(option, v)

        return ret




import os
import re

import testdata

from stockton import cli
from stockton.path import Filepath, Dirpath
from stockton.concur.formats.postfix import Main, SMTPd, Master
from stockton.interface import Postfix, Spam, Razor, Pyzor, DKIM, SRS, SMTP, DCC

from . import TestCase as BaseTestCase, Stockton


class TestCase(BaseTestCase):
#     def setUp(self):
#         super(TestCase, self).setUp()

    @classmethod
    def setup_env(cls):
        p = Postfix()
        if not p.main_f.exists():
            s = Stockton("prepare")
            r = s.run()

    def setup_domain(self, domain):
        s = Stockton("configure-recv")
        r = s.run("--mailserver=mail.example.com")

        s = Stockton("add-domain")
        r = s.run("{} --proxy-email=dest@dest.com".format(domain))


class PrepareTest(TestCase):
    @classmethod
    def setup_env(cls):
        cli.purge("postfix")

    def test_run(self):
        p = Postfix()
        d = p.config_d
        self.assertFalse(d.exists())

        s = Stockton("prepare")
        r = s.run()

        f = p.main_f
        self.assertTrue(f.exists())


class ConfigureRecvTest(TestCase):
    @classmethod
    def setup_env(cls):
        super(ConfigureRecvTest, cls).setup_env()

        s = Stockton("configure-recv")
        r = s.run("--mailserver=mail.example.com")

    def test_recv(self):
        p = Postfix()

        # make some changes to main
        m = p.main()
        m.update(
            ("myhostname", "foobar.com")
        )
        m.save()

        f = p.main_f
        self.assertTrue(f.contains("foobar.com"))

        # re-run
        s = Stockton("configure-recv")
        r = s.run("--mailserver=mail.example.com")
        f = p.main_f
        self.assertFalse(f.contains("foobar.com"))
        self.assertTrue(p.is_running())


class ConfigureSendTest(TestCase):
    def test_send(self):
        p = Postfix()
        s = Stockton("configure-send")
        r = s.run("--mailserver=mail.example.com")
        self.assertTrue(p.is_running())


class ConfigureDKIMTest(TestCase):
    @classmethod
    def setup_env(cls):
        dk = DKIM()
        dk.uninstall()
        super(ConfigureDKIMTest, cls).setup_env()

    def test_dkim(self):
        #self.test_recv() # we need a configured for receive postfix
        self.setup_domain("example.com")

        s = Stockton("configure-dkim")
        r = s.run()

        dk = DKIM()
        p = Postfix()
        self.assertTrue(dk.config_d.exists())
        self.assertEqual(1, dk.keytable_f.lc())
        self.assertEqual(1, dk.signingtable_f.lc())
        self.assertTrue("*.example.com" in dk.trustedhosts_f.lines())
        self.assertTrue(dk.is_running())
        self.assertTrue(p.is_running())


class ConfigureSRSTest(TestCase):
    def test_srs(self):
        s = Stockton("configure-srs")
        r = s.run("")

        sr = SRS()
        self.assertTrue(sr.is_running())


class ConfigureSpamLibsTest(TestCase):
    def test_dcc(self):
        sp = Spam()
        sp.uninstall()

        dcc = DCC()
        dcc.uninstall()
        self.assertFalse(dcc.exists())

        self.setup_domain("example.com")

        with self.assertRaises(RuntimeError):
            s = Stockton("configure-dcc")
            r = s.run()

        s = Stockton("lockdown-spam")
        r = s.run()

        s = Stockton("configure-dcc")
        r = s.run()

        self.assertTrue("DCC_CHECK" in dcc.test())

    def test_pyzor(self):
        sp = Spam()
        sp.uninstall()
        p = Pyzor()
        p.uninstall()
        self.assertFalse(p.exists())

        self.setup_domain("example.com")

        with self.assertRaises(RuntimeError):
            s = Stockton("configure-pyzor")
            r = s.run()

        s = Stockton("lockdown-spam")
        r = s.run()

        s = Stockton("configure-pyzor")
        r = s.run()

        self.assertTrue("pyzor is available" in p.test())

    def test_razor(self):
        sp = Spam()
        sp.uninstall()
        ra = Razor()
        ra.uninstall()
        self.assertFalse(ra.exists())

        self.setup_domain("example.com")

        with self.assertRaises(RuntimeError):
            s = Stockton("configure-razor")
            r = s.run()

        s = Stockton("lockdown-spam")
        r = s.run()

        s = Stockton("configure-razor")
        r = s.run()

        self.assertTrue("Found razorhome: {}".format(ra.home_d) in ra.test())


class DomainTest(TestCase):
#     @classmethod
#     def setup_env(cls):
#         dk = DKIM()
#         dk.uninstall()
#         super(DomainTest, cls).setup_env()

    def test_delete_domain(self):
        self.setup_domain("example.com")
        s = Stockton("configure_dkim")
        r = s.run()

        s = Stockton("add-domain")
        r = s.run("todelete.com --proxy-email=todelete@dest.com")

        s = Stockton("delete-domain")
        r = s.run("todelete.com")

        # verify
        af = Filepath("/etc/postfix/virtual/addresses/todelete.com")
        self.assertFalse(af.exists())

        df = Filepath("/etc/postfix/virtual/domains")
        self.assertFalse(df.contains("todelete.com"))

        opendkim_d = Dirpath("/etc/opendkim")
        signingtable_f = Filepath(opendkim_d, "SigningTable")
        self.assertFalse(signingtable_f.contains("todelete.com"))

        trustedhosts_f = Filepath(opendkim_d, "TrustedHosts")
        self.assertFalse(trustedhosts_f.contains("todelete.com"))

    def test_add_domain_proxy_file_smtp(self):
        self.setup_domain("example.com")
        s = Stockton("configure-send")
        r = s.run("--mailserver=mail.example.com")

        proxy_addresses = testdata.create_file("foo.com", [
            "one@foo.com                foo@dest1.com",
            "two@foo.com                foo@dest2.com",
            "three@foo.com              foo@dest3.com",
            "",
        ])

        s = Stockton("add-domain")
        r = s.run("foo.com --proxy-file={} --smtp-password=12345".format(proxy_addresses))

        cli.package("cyrus-clients-2.4") # for smtptest
        r = cli.run(
            "echo QUIT | smtptest -a smtp@foo.com -w 12345 -t /etc/postfix/certs/mail.example.com.pem -p 587 localhost",
            capture_output=True
        )
        self.assertRegexpMatches(r, "235[^A]+Authentication\s+successful")

    def test_update_domain_proxy(self):
        # to make sure this works completely, we completely remove postfix
        p = Postfix()
        p.reset()

        f = testdata.create_file("foo.com", [
            "one@foo.com                foo@dest.com",
            "",
        ])

        s = Stockton("add-domain")
        r = s.run("{} --proxy-file={}".format(f.name, f))
        m = Main(prototype_path=Main.dest_path)
        proxy_path = re.search("^hash:([/a-z]+?foo\.com)$", m["virtual_alias_maps"].val, re.M).group(1)
        proxy_path_f = Filepath(proxy_path)
        self.assertTrue("one@foo.com" in proxy_path_f.contents())

        s = Stockton("update-domain-proxy")
        f = testdata.create_file("foo.com", [
            "two@bar.com                bar@dest.com",
            "",
        ])
        r = s.run("{} --proxy-file={}".format(f.name, f))
        m = Main(prototype_path=Main.dest_path)
        proxy_path = re.search("^hash:([/a-z]+?foo\.com)$", m["virtual_alias_maps"].val, re.M).group(1)
        proxy_path_f = Filepath(proxy_path)
        self.assertFalse("one@foo.com" in proxy_path_f.contents())
        self.assertTrue("two@bar.com" in proxy_path_f.contents())

    def test_add_domain_proxy_file_no_smtp(self):

        # to make sure this works completely, we completely remove postfix
        p = Postfix()
        p.reset()

        s = Stockton("add-domain")
        proxy_domains = testdata.create_files({
            "foo.com": "\n".join([
                "one@foo.com                foo@dest.com",
                "two@foo.com                foo@dest.com",
                "three@foo.com              foo@dest.com",
                "",
            ]),
            "bar.com": "\n".join([
                "one@bar.com                bar@dest.com",
                "two@bar.com                bar@dest.com",
                "three@bar.com              bar@dest.com",
                "",
            ]),
        })

        for f in proxy_domains:
            r = s.run("{} --proxy-file={}".format(f.name, f))

        contents = "\n".join([
            "@foo.com                   foo@dest.com",
            "",
        ])
        f = testdata.create_file("foo.com", contents)

        r = s.run("{} --proxy-file={}".format(f.name, f))
        foo = Filepath("/etc/postfix/virtual/addresses/foo.com")
        self.assertEqual(contents, foo.contents())

        # Let's check structure because I was having a lot of problems with getting
        # the structure...

        # domains file should have 2 domains in it
        domains_f = Filepath("/etc/postfix/virtual/domains")
        self.assertEqual(2, domains_f.lc())
        self.assertTrue(domains_f.contains("^foo\.com$"))

        m = Main(prototype_path=Main.dest_path)
        # we can't guarrantee foo, bar order so we match one line at a time
        self.assertTrue(re.search("^hash:[/a-z]+?(bar|foo)\.com,", m["virtual_alias_maps"].val, re.M))
        self.assertTrue(re.search("^\s+hash:[/a-z]+?(bar|foo)\.com$", m["virtual_alias_maps"].val, re.M))

    def test_add_domains(self):
        s = Stockton("add-domains")

        # a file with different domains
        proxy_domains = testdata.create_files({
            "foo2.com.txt": [
                "one@foo3.com                foo@dest.com",
                "one@bar3.com                bar@dest.com",
                "",
            ],
        })
        with self.assertRaises(RuntimeError):
            r = s.run("--proxy-domains={}".format(proxy_domains))

        # 2 files with the same domain
        proxy_domains = testdata.create_files({
            "foo2.com.txt": [
                "one@foo2.com                foo@dest.com",
                "",
            ],
            "bar2.com": [
                "one@foo2.com                bar@dest.com",
                "",
            ],
        })
        with self.assertRaises(RuntimeError):
            r = s.run("--proxy-domains={}".format(proxy_domains))

        proxy_domains = testdata.create_files({
            "foo.com.txt": [
                "one@foo.com                foo@dest.com",
                "two@foo.com                foo@dest.com",
                "three@foo.com              foo@dest.com",
                "",
            ],
            "bar.com": [
                "one@bar.com                bar@dest.com",
                "two@bar.com                bar@dest.com",
                "three@bar.com              bar@dest.com",
                "",
            ],
            "che.org.txt": [
                "@che.org                   che@dest.com",
                "",
            ],
        })

        r = s.run("--proxy-domains={} --smtp-password=1234".format(proxy_domains))
        self.assertTrue("Adding domain foo.com" in r)
        self.assertTrue("Adding domain bar.com" in r)
        self.assertTrue("Adding domain che.org" in r)

    def test_add_domain_domain(self):
        """pass in the domain and the proxy"""
        s = Stockton("configure-dkim")
        r = s.run()

        p = Postfix()
        dk = DKIM()

        s = Stockton("add-domain")
        one_f = p.address("one.com")
        two_f = p.address("two.com")
        domains_f = p.domains_f

        r = s.run("one.com --proxy-email=one@dest.com")
        self.assertTrue(one_f.contains("^@one.com"))
        self.assertTrue(domains_f.contains("one.com"))

        r = s.run("two.com --proxy-email=two@dest.com")
        self.assertTrue(one_f.contains("^@one.com"))
        self.assertTrue(domains_f.contains("one.com"))
        self.assertTrue(two_f.contains("two@dest.com"))
        self.assertTrue(domains_f.contains("two.com"))

        opendkim_d = dk.config_d
        keytable_f = dk.keytable_f
        signingtable_f = dk.signingtable_f
        trustedhosts_f = dk.trustedhosts_f

        self.assertTrue(keytable_f.contains("one.com"))
        self.assertTrue(signingtable_f.contains("one.com"))
        self.assertTrue(keytable_f.contains("two.com"))
        self.assertTrue(signingtable_f.contains("two.com"))
        self.assertTrue("*.one.com" in trustedhosts_f.lines())
        self.assertTrue("*.two.com" in trustedhosts_f.lines())

    def test_add_domain_smtp(self):
        s = Stockton("configure-send")
        r = s.run("--mailserver=mail.example.com")

        s = Stockton("add-domain")
        r = s.run("one.com --proxy-email=one@dest.com --smtp-password=1234")

        cli.package("cyrus-clients-2.4") # for smtptest
        r = cli.run(
            "echo QUIT | smtptest -a smtp@one.com -w 1234 -t /etc/postfix/certs/mail.example.com.pem -p 587 localhost"
        )
        self.assertRegexpMatches(r, "235[^A]+Authentication\s+successful")

        r = cli.run(
            "echo QUIT | smtptest -a smtp@one.com -w 9876 -t /etc/postfix/certs/mail.example.com.pem -p 587 localhost"
        )
        self.assertRegexpMatches(r, "535[^E]+Error:\s+authentication\s+failed")

        r = cli.run(
            "echo QUIT | smtptest -a foo@one.com -w 1234 -t /etc/postfix/certs/mail.example.com.pem -p 587 localhost"
        )
        self.assertRegexpMatches(r, "535[^E]+Error:\s+authentication\s+failed")




#         cli.package("cyrus-clients-2.4") # for smtptest
#
#         s = Stockton("configure-send")
#         arg_str = "example.com --mailserver=mail.example.com --smtp-password=1234"
#         r = s.run(arg_str)
# 
#         r = cli.run(
#             "echo QUIT | smtptest -a smtp@example.com -w 1234 -t /etc/postfix/certs/mail.example.com.pem -p 587 localhost",
#             capture_output=True
#         )
#         self.assertRegexpMatches(r, "235[^A]+Authentication\s+successful")
# 
#         r = cli.run(
#             "echo QUIT | smtptest -a smtp@example.com -w 9876 -t /etc/postfix/certs/mail.example.com.pem -p 587 localhost",
#             capture_output=True
#         )
#         self.assertRegexpMatches(r, "535[^E]+Error:\s+authentication\s+failed")
# 
#         r = cli.run(
#             "echo QUIT | smtptest -a foo@example.com -w 1234 -t /etc/postfix/certs/mail.example.com.pem -p 587 localhost",
#             capture_output=True
#         )
#         self.assertRegexpMatches(r, "535[^E]+Error:\s+authentication\s+failed")
# 
# #         r = cli.run(
# #             "echo QUIT | smtptest -a smtp -w 1234 -t /etc/postfix/certs/example.com.pem -p 587 localhost",
# #             capture_output=True
# #         )
# #         self.assertRegexpMatches(r, "535[^E]+Error:\s+authentication\s+failed")


class LockdownTest(TestCase):
    def test_all(self):
        s = Stockton("lockdown")
        r = s.run("--mailserver=mail.example.com")

        p = Postfix()
        sp = Spam()
        self.assertTrue(p.is_running())
        self.assertTrue(p.helo_f.exists())
        self.assertTrue(sp.is_running())

    def test_spam(self):
        s = Stockton("lockdown-spam")
        r = s.run()

        sp = Spam()
        self.assertTrue(sp.is_running())

    def test_postfix(self):
        self.setup_domain("example.com")

        s = Stockton("lockdown-postfix")
        r = s.run("--mailserver=mail.example.com")

        p = Postfix()
        self.assertTrue(p.is_running())
        self.assertTrue(p.helo_f.exists())


class InstallationTest(TestCase):
    @classmethod
    def setup_env(cls):
        #super(ConfigureDKIMTest, cls).setup_env()
        pass

    def test_idempotence(self):
        self.test_uninstall()
        self.test_install()

        p = Postfix()
        dk = DKIM()
        sp = Spam()
        sm = SMTP()

        main_hash = p.main_f.checksum
        master_hash = p.master_f.checksum
        dk_hash = dk.config_f.checksum
        sp_config_hash = sp.config_f.checksum
        sp_local_hash = sp.local_f.checksum
        sm_config_hash = sm.config_f.checksum

        self.test_install()

        # just to make sure there is no caching issues
        p = Postfix()
        dk = DKIM()
        sp = Spam()
        sm = SMTP()

        self.assertEqual(main_hash, p.main_f.checksum)
        self.assertEqual(master_hash, p.master_f.checksum)
        self.assertEqual(dk_hash, dk.config_f.checksum)
        self.assertEqual(sp_config_hash, sp.config_f.checksum)
        self.assertEqual(sp_local_hash, sp.local_f.checksum)
        self.assertEqual(sm_config_hash, sm.config_f.checksum)

    def test_uninstall(self):
        s = Stockton("uninstall")
        r = s.run()

        p = Postfix()
        self.assertFalse(p.is_running())
        self.assertFalse(p.exists())

        dk = DKIM()
        self.assertFalse(dk.is_running())
        self.assertFalse(dk.exists())

        sr = SRS()
        self.assertFalse(sr.is_running())

        sp = Spam()
        self.assertFalse(sp.is_running())
        self.assertFalse(sp.exists())

        sm = SMTP()
        self.assertFalse(sm.exists())

    def test_install_proxy_domains(self):
        s = Stockton("install")
        proxy_domains = testdata.create_files({
            "pd1.com.txt": [
                "one@pd1.com                foo@dest.com",
                "",
            ],
            "pd2.com": [
                "one@pd2.com                bar@dest.com",
                "",
            ],
        })

        r = s.run("--mailserver={} --proxy-domains={} --smtp-password=1234".format(
            "mail.example.com",
            proxy_domains
        ))
        # if there was no error than yay, it worked

    def test_install(self):
        s = Stockton("install")
        arg_str = " ".join([
            "--mailserver=mail.example.com",
            "--smtp-password=1234",
            "--proxy-email=d@dest.com",
            "front.com"
        ])
        r = s.run(arg_str)

        p = Postfix()
        self.assertTrue(p.is_running())

        dk = DKIM()
        self.assertTrue(dk.is_running())

        sr = SRS()
        self.assertTrue(sr.is_running())

        sp = Spam()
        self.assertTrue(sp.is_running())




"""
Generic starter config parsers for a variety of different file formats

https://en.wikipedia.org/wiki/Configuration_file
http://stackoverflow.com/questions/1925305/best-config-file-format

"""
import re

import base


class EqualOption(base.ConfigOption):
    def format_set(self, name, divider, val):
        option_set_format = "{} {} {}" # key = val
        s = option_set_format.format(name, divider, val)
        return s


class EqualConfig(base.Config):
    """Configuration files that have a format like

    name = val

    type of configuration
    """
    option_divider = "="
    option_class = EqualOption


class ColonOption(base.ConfigOption):
    def format_set(self, name, divider, val):
        option_set_format = "{}{} {}" # key: val
        s = option_set_format.format(name, divider, val)
        return s


class ColonConfig(base.Config):
    """Configuration files that have a format like

    name: val

    type of configuration
    """
    option_divider = ":"
    option_class = ColonOption


class SpaceOption(base.ConfigOption):
    def format_set(self, name, divider, val):
        option_set_format = "{}{}" # key val
        s = option_set_format.format(
            name.ljust(max(self.config.option_buffer, len(name) + 1), " "),
            val
        )
        return s


class SpaceConfig(base.Config):
    """Configuration files that have a format like

    name            val

    type of configuration
    """
    option_divider = "\s"
    option_buffer = 24
    option_class = SpaceOption




import re

from captain import echo

from .. import cli
from .postfix import Postfix
from ..path import Dirpath, Filepath, Sentinal
#from ..concur.formats.opendkim import OpenDKIM
from ..concur.formats.spamassassin import SpamAssassin, Local
from ..concur.formats.generic import EqualConfig


class Spam(object):
    """Installs and adds configuration hooks for SpamAssassin

    https://spamassassin.apache.org/full/3.4.x/doc/
    http://wiki.apache.org/spamassassin/ImproveAccuracy
    https://aikar.co/2014/09/05/filtering-spam-forwarding-email-postfixspamassassin/
    https://github.com/apache/spamassassin
    """

    perl_packages = [
        "libgeo-ip-perl",
        "libdigest-sha-perl",
        "libdbi-perl",
        "libio-socket-ip-perl",
        "libencode-detect-perl",
        "libnet-patricia-perl",
        "libmail-dkim-perl",
        "libmail-spf-perl",
    ]

    @property
    def user(self):
        return "debian-spamd"

    @property
    def home_d(self):
        # http://askubuntu.com/questions/410244/a-command-to-list-all-users-and-how-to-add-delete-modify-users
        username = self.user
        path = cli.run(
            'grep "{}" /etc/passwd | cut -f6 -d":"'.format(username),
            capture_output=True
        ).strip()

        if not path:
            raise ValueError("could not find home directory for user {}".format(username))

        return Dirpath(path)

    @property
    def config_f(self):
        return Filepath(SpamAssassin.dest_path)

    @property
    def local_f(self):
        return Filepath(Local.dest_path)

    def base_configs(self):
        return [self.config_f, self.local_f]

    def pre_config(self, basename):
        """return configuration for one of the pre configuration files of spam assassin

        :param basename: something like v310
        :returns: the configuration instance
        """
        if not basename.endswith(".pre"):
            basename = "{}.pre".format(basename)
        f = Filepath(Filepath(Local.dest_path).directory, basename)
        return Local(prototype_path=str(f), dest_path=str(f))

    def config(self, path=SpamAssassin.dest_path):
        return SpamAssassin(prototype_path=path)

    def local(self, path=Local.dest_path):
        return Local(prototype_path=path)

    def start(self):
        try:
            o = cli.run("/etc/init.d/spamassassin start")
        except RuntimeError as e:
            if not re.search("already\s+running", str(e), re.I):
                raise

    def restart(self):
        if self.is_running():
            cli.run("/etc/init.d/spamassassin restart")
        else:
            self.start()

    def stop(self):
        cli.run("/etc/init.d/spamassassin stop")

    def is_running(self):
        ret = True
        try:
            cli.run("/etc/init.d/spamassassin status")
        except RuntimeError:
            ret = False
        return ret

    def exists(self):
        ret = True
        try:
            #self.stop()
            cli.run("/etc/init.d/spamassassin status")
        except cli.RunError as e:
            ret = not e.is_missing()
        return ret

    def lint(self):
        """Return the output from checking SA's configuration"""
        return cli.run("spamassassin -D --lint")

    def install(self):
        # make sure the packages are installed
        cli.package("spamassassin", "spamc")

        # these help make SA work better
        cli.package("libgeoip-dev", "libssl-dev") # we don't remove these in uninstall()
        cli.package(*self.perl_packages)

        # add the spam user
        # update 4-16-2016 let's just use debian-spamd and /var/lib/spamassassin
        #cli.run("adduser --disabled-login --disabled-password --gecos "" spamd")
        if not self.home_d.exists():
            raise ValueError("Home directory {} does not exist".format(self.home_d))

    def uninstall(self):
        try:
            self.stop()
        except cli.RunError as e:
            if not e.is_missing():
                raise

        cli.purge("spamassassin", "spamc")
        cli.purge(*self.perl_packages)


class Razor(object):
    """Installs razor

    http://razor.sourceforge.net/
    https://digitalenvelopes.email/blog/debian-integrate-razor-spamassassin/
    https://wiki.apache.org/spamassassin/RazorSiteWide
    https://spamassassin.apache.org/full/3.4.x/doc/Mail_SpamAssassin_Plugin_Razor2.html
    https://wiki.apache.org/spamassassin/UsingRazor

    https://wiki.apache.org/spamassassin/RazorHowToTell
    to test --
        razor-check -d < /usr/share/doc/spamassassin/examples/sample-spam.txt
        spamassassin -D razor2 2>&1 < /usr/share/doc/spamassassin/examples/sample-spam.txt

        This is the one I've been using recently to make sure everything is working:
        echo "test" | spamassassin -t -D razor2
    """
    @property
    def home_d(self):
        return Dirpath("/etc/razor")

    @property
    def config_f(self):
        return Filepath(self.home_d, "razor-agent.conf")

    def test(self):
        r = cli.run("spamassassin -D razor2 < /usr/share/doc/spamassassin/examples/sample-spam.txt")
        return r

    def config(self, path=""):
        if not path:
            path = self.config_f
        path = str(path)
        dest_path = str(self.config_f)
        return EqualConfig(dest_path=dest_path, prototype_path=path)

    def start(self):
        pass

    def restart(self):
        pass

    def stop(self):
        pass

    def is_running(self):
        return False

    def exists(self):
        ret = True
        try:
            cli.run("which razor-admin")
        except cli.RunError as e:
            ret = False
        return ret

    def install(self):
        # make sure the packages are installed
        cli.package("razor")

        cli.run("razor-admin -d -home=\"{}\" -create".format(self.home_d))
        cli.run("razor-admin -d -home=\"{}\" -register".format(self.home_d))
        # razor-admin -d -home=/etc/razor -create
        # razor-admin -d -home=/etc/razor -register

    def uninstall(self):
        cli.purge("razor")


class Pyzor(object):
    """Installs pyzor

    https://digitalenvelopes.email/blog/debian-integrate-pyzor-spamassassin/
    https://wiki.apache.org/spamassassin/UsingPyzor

    to test:
        echo "test" | spamassassin -D pyzor
        spamassassin -D pyzor < /usr/share/doc/spamassassin/examples/sample-spam.txt
    """
    def test(self):
        r = cli.run("spamassassin -D pyzor < /usr/share/doc/spamassassin/examples/sample-spam.txt")
        return r

    def start(self):
        pass

    def restart(self):
        pass

    def stop(self):
        pass

    def is_running(self):
        return False

    def ping(self):
        output = cli.run("pyzor ping")
        return "200" in output

    def exists(self):
        ret = True
        try:
            cli.run("which pyzor")
        except cli.RunError as e:
            ret = False
        return ret

    def install(self):
        cli.package("pyzor")

        cli.run("pyzor discover")

        if not self.ping():
            raise IOError("Pyzor ping was unsuccessful")

    def uninstall(self):
        cli.purge("pyzor")


class DCC(object):
    """Install DCC

    http://forum.directadmin.com/showthread.php?t=53179&s=5d1d3471d0ca0a99b019f457758f86c4&p=272841#post272841
    https://www.dcc-servers.net/dcc/FAQ.html
    https://wiki.apache.org/spamassassin/InstallingDCC

    to test:
        spamassassin -D DCC < /usr/share/doc/spamassassin/examples/sample-spam.txt
    """
    @property
    def user(self):
        return "mail"

    @property
    def home_d(self):
        return Dirpath("/var/lib/dcc")

    def test(self):
        r = cli.run("spamassassin -D DCC < /usr/share/doc/spamassassin/examples/sample-spam.txt")
        return r

    def start(self):
        pass

    def restart(self):
        pass

    def stop(self):
        pass

    def is_running(self):
        return False

    def exists(self):
        ret = True
        try:
            cli.run("which cdcc")
        except cli.RunError as e:
            ret = False
        return ret

    def install(self):
        tmpdir = Dirpath.get_temp()
        tar_name = "dcc-dccproc.tar.Z"
        tar_src = "http://www.dcc-servers.net/dcc/source/dcc-dccproc.tar.Z"

        tar_f = Filepath(tmpdir, tar_name)
        if not tar_f.exists():
            cli.run("wget -O {} {}".format(tar_name, tar_src), cwd=tmpdir.path)

        # Build and install.
        cli.run("tar xzvf {}".format(tar_name), cwd=tmpdir.path)
        build_d = tmpdir.descendant(r"dcc-dccproc-")
        run_d = self.home_d

        cli.run(
            "".join([
                "./configure "
                "--bindir=$(PREFIX)/bin ",
                "--libexecdir=$(PREFIX)/lib/dcc ",
                "--mandir=$(PREFIX)/man ",
                "--homedir={} ".format(run_d),
                "--with-uid={} ".format(self.user),
                "--with-gid={}".format(self.user)
            ]),
            cwd=build_d.path
        )
        cli.run("make", cwd=build_d.path)
        cli.run("make install", cwd=build_d.path)
        run_d.chown(self.user, R=True)
        run_d.chgrp(self.user, R=True)

    def uninstall(self):
        #cli.purge("razor")
        for path in ["/lib/dcc", "/var/lib/dcc"]:
            d = Dirpath(path)
            d.delete()

        # clean up executable files
        d = Dirpath("/bin")
        d.delete_files(r"^c?dcc")

        # delete man
        d = Dirpath("/man/man8")
        d.delete_files(r"^c?dcc")

